@@ -13,8 +13,10 @@ import java.util.Hashtable;
 import java.util.List;
 import java.util.Vector;
 
+import org.gnu.glib.Error;
 import org.gnu.glib.GObject;
-import org.gnu.glib.Handle;
+import org.gnu.javagnome.Handle;
+import org.gnu.javagnome.Struct;
 
 /**
  * ConfClient provides a client-side cache for a specified list of directories
@@ -59,9 +61,9 @@ public class ConfClient extends GObject {
 	 * @throws ConfException
 	 */
 	public void addDirectory(String dir, ConfClientPreloadType type) throws ConfException {
-		int err = -1;
-		gconf_client_add_dir(getHandle(), dir, type.getValue(), new int[] {err});
-		checkError(err);
+		Handle errorHandle = Struct.getNullHandle();
+		gconf_client_add_dir(getHandle(), dir, type.getValue(), errorHandle);
+		checkError(errorHandle);
 	}
 
 	/**
@@ -70,9 +72,9 @@ public class ConfClient extends GObject {
 	 * @throws ConfException
 	 */
 	public void removeDirectory(String dir) throws ConfException {
-		int err = -1;
-		gconf_client_remove_dir(getHandle(), dir, new int[] {err});
-		checkError(err);
+		Handle errorHandle = Struct.getNullHandle();
+		gconf_client_remove_dir(getHandle(), dir, errorHandle);
+		checkError(errorHandle);
 	}
 
 	
@@ -82,13 +84,15 @@ public class ConfClient extends GObject {
 	 * @see org.gnu.gconf.ConfClientListener
 	 */
 	public void addListener(ConfClientListener listener, String nameSpace)  throws ConfException {
-		int err = -1;
-		int id = gconf_client_notify_add(getHandle(), nameSpace, new int[] {err});
-		checkError(err);
+		Handle errorHandle = Struct.getNullHandle();
+		int id = gconf_client_notify_add(getHandle(), nameSpace, errorHandle);
+		checkError(errorHandle);
 		
+        NotifyData nd = null;
 		if (null == listeners)
 				listeners = new Hashtable();
-		NotifyData nd = (NotifyData)listeners.get(nameSpace);
+        else
+            nd = (NotifyData)listeners.get(nameSpace);
 
 		if (null == nd) {
 			nd = new NotifyData();
@@ -148,9 +152,9 @@ public class ConfClient extends GObject {
 	 * @throws ConfException
 	 */
 	public void preload(String directory, ConfClientPreloadType type) throws ConfException {
-		int err = -1;
-		gconf_client_preload(getHandle(), directory, type.getValue(), new int[] {err});
-		checkError(err);
+        Handle errorHandle = Struct.getNullHandle();
+		gconf_client_preload(getHandle(), directory, type.getValue(), errorHandle);
+		checkError(errorHandle);
 	}
 
 	/**
@@ -160,9 +164,9 @@ public class ConfClient extends GObject {
 	 * @throws ConfException
 	 */
 	public void set(String key, ConfValue value) throws ConfException {
-		int err = -1;
-		gconf_client_set(getHandle(), key, value.getHandle(), new int[] {err});
-		checkError(err);
+		Handle errorHandle = Struct.getNullHandle();
+		gconf_client_set(getHandle(), key, value.getHandle(), errorHandle);
+		checkError(errorHandle);
 	}
 	
 	/**
@@ -171,9 +175,9 @@ public class ConfClient extends GObject {
 	 * @throws ConfException
 	 */
 	public ConfValue get(String key) throws ConfException {
-		int err = -1;
-		Handle hndl = gconf_client_get(getHandle(), key, new int[] {err});
-		checkError(err);
+		Handle errorHandle = Struct.getNullHandle();
+		Handle hndl = gconf_client_get(getHandle(), key, errorHandle);
+		checkError(errorHandle);
 		return new ConfValue(hndl);
 	}
 	
@@ -185,9 +189,9 @@ public class ConfClient extends GObject {
 	 * @throws ConfException
 	 */
 	public ConfEntry getEntry(String key, String locale, boolean useSchemaDefault) throws ConfException {
-		int err = -1;
-		Handle hndl = gconf_client_get_entry(getHandle(), key, locale, useSchemaDefault, new int[] {err});
-		checkError(err);
+		Handle errorHandle = Struct.getNullHandle();
+		Handle hndl = gconf_client_get_entry(getHandle(), key, locale, useSchemaDefault, errorHandle);
+		checkError(errorHandle);
 		return new ConfEntry(hndl);
 	}
 	
@@ -199,9 +203,9 @@ public class ConfClient extends GObject {
 	 * @throws ConfException
 	 */
 	public ConfValue getDefaultFromSchema(String key) throws ConfException {
-		int err = -1;
-		Handle hndl = gconf_client_get_default_from_schema(getHandle(), key, new int[] {err});
-		checkError(err);
+		Handle errorHandle = Struct.getNullHandle();
+		Handle hndl = gconf_client_get_default_from_schema(getHandle(), key, errorHandle);
+		checkError(errorHandle);
 		return new ConfValue(hndl);
 	}
 	
@@ -211,9 +215,9 @@ public class ConfClient extends GObject {
 	 * @throws ConfException
 	 */
 	public boolean unset(String key) throws ConfException {
-		int err = -1;
-		boolean ret = gconf_client_unset(getHandle(), key, new int[] {err});
-		checkError(err);
+		Handle errorHandle = Struct.getNullHandle();
+		boolean ret = gconf_client_unset(getHandle(), key, errorHandle);
+		checkError(errorHandle);
 		return ret;
 	}
 	
@@ -223,9 +227,9 @@ public class ConfClient extends GObject {
 	 * @throws ConfException
 	 */
 	public List getAllEntries(String dir) throws ConfException {
-		int err = -1;
-		Handle [] array = gconf_client_all_entries(getHandle(), dir, new int[] {err});
-		checkError(err);
+		Handle errorHandle = Struct.getNullHandle();
+		Handle [] array = gconf_client_all_entries(getHandle(), dir, errorHandle);
+		checkError(errorHandle);
 		if (array == null)
 			return null;
 		List ret = new ArrayList();
@@ -241,9 +245,9 @@ public class ConfClient extends GObject {
 	 * @throws ConfException
 	 */
 	public List getAllDirs(String dir) throws ConfException {
-		int err = -1;
-		String [] array = gconf_client_all_dirs(getHandle(), dir, new int[] {err});
-		checkError(err);
+		Handle errorHandle = Struct.getNullHandle();
+		String [] array = gconf_client_all_dirs(getHandle(), dir, errorHandle);
+		checkError(errorHandle);
 		if (array == null)
 			return null;
 		List ret = new ArrayList();
@@ -259,9 +263,9 @@ public class ConfClient extends GObject {
 	 * @throws ConfException
 	 */
 	public void suggestSync() throws ConfException {
-		int err = -1;
-		gconf_client_suggest_sync(getHandle(), new int[] {err});
-		checkError(err);
+		Handle errorHandle = Struct.getNullHandle();
+		gconf_client_suggest_sync(getHandle(), errorHandle);
+		checkError(errorHandle);
 	}
 	
 	/**
@@ -270,9 +274,9 @@ public class ConfClient extends GObject {
 	 * @throws ConfException
 	 */
 	public boolean dirExists(String dir) throws ConfException {
-		int err = -1;
-		boolean ret = gconf_client_dir_exists(getHandle(), dir, new int[] {err});
-		checkError(err);
+		Handle errorHandle = Struct.getNullHandle();
+		boolean ret = gconf_client_dir_exists(getHandle(), dir, errorHandle);
+		checkError(errorHandle);
 		return ret;
 	}
 	
@@ -282,9 +286,9 @@ public class ConfClient extends GObject {
 	 * @throws ConfException
 	 */
 	public double getDouble(String key) throws ConfException {
-		int err = -1;
-		double ret = gconf_client_get_float(getHandle(), key, new int[] {err});
-		checkError(err);
+		Handle errorHandle = Struct.getNullHandle();
+		double ret = gconf_client_get_float(getHandle(), key, errorHandle);
+		checkError(errorHandle);
 		return ret;
 	}
 	
@@ -294,9 +298,9 @@ public class ConfClient extends GObject {
 	 * @throws ConfException
 	 */
 	public int getInt(String key) throws ConfException {
-		int err = -1;
-		int ret = gconf_client_get_int(getHandle(), key, new int[] {err});
-		checkError(err);
+		Handle errorHandle = Struct.getNullHandle();
+		int ret = gconf_client_get_int(getHandle(), key, errorHandle);
+		checkError(errorHandle);
 		return ret;
 	}
 	
@@ -306,9 +310,9 @@ public class ConfClient extends GObject {
 	 * @throws ConfException
 	 */
 	public String getString(String key) throws ConfException {
-		int err = -1;
-		String ret = gconf_client_get_string(getHandle(), key, new int[] {err});
-		checkError(err);
+		Handle errorHandle = Struct.getNullHandle();
+		String ret = gconf_client_get_string(getHandle(), key, errorHandle);
+		checkError(errorHandle);
 		return ret;
 	}
 	
@@ -318,9 +322,9 @@ public class ConfClient extends GObject {
 	 * @throws ConfException
 	 */
 	public boolean getBoolean(String key) throws ConfException {
-		int err = -1;
-		boolean ret = gconf_client_get_bool(getHandle(), key, new int[] {err});
-		checkError(err);
+		Handle errorHandle = Struct.getNullHandle();
+		boolean ret = gconf_client_get_bool(getHandle(), key, errorHandle);
+		checkError(errorHandle);
 		return ret;
 	}
 	
@@ -330,9 +334,9 @@ public class ConfClient extends GObject {
 	 * @throws ConfException
 	 */
 	public ConfSchema getSchema(String key) throws ConfException {
-		int err = -1;
-		Handle ret = gconf_client_get_schema(getHandle(), key, new int[] {err});
-		checkError(err);
+		Handle errorHandle = Struct.getNullHandle();
+		Handle ret = gconf_client_get_schema(getHandle(), key, errorHandle);
+		checkError(errorHandle);
 		return new ConfSchema(ret);
 	}
 	
@@ -344,9 +348,9 @@ public class ConfClient extends GObject {
 	 * @throws ConfException
 	 */
 	public Object[] getList(String key, ConfValueType listType) throws ConfException {
-		int err = -1;
-		Object [] list = gconf_client_get_list(getHandle(), key, listType.getValue(), new int[] {err});
-		checkError(err);
+		Handle errorHandle = Struct.getNullHandle();
+		Object [] list = gconf_client_get_list(getHandle(), key, listType.getValue(), errorHandle);
+		checkError(errorHandle);
 		return list;
 	}
 	
@@ -358,9 +362,9 @@ public class ConfClient extends GObject {
 	 * @throws ConfException
 	 */
 	public boolean setDouble(String key, double value) throws ConfException {
-		int err = -1;
-		boolean ret = gconf_client_set_float(getHandle(), key, value, new int[] {err});
-		checkError(err);
+		Handle errorHandle = Struct.getNullHandle();
+		boolean ret = gconf_client_set_float(getHandle(), key, value, errorHandle);
+		checkError(errorHandle);
 		return ret;
 	}
 	
@@ -372,9 +376,9 @@ public class ConfClient extends GObject {
 	 * @throws ConfException
 	 */
 	public boolean setInt(String key, int value) throws ConfException {
-		int err = -1;
-		boolean ret = gconf_client_set_int(getHandle(), key, value, new int[] {err});
-		checkError(err);
+		Handle errorHandle = Struct.getNullHandle();
+		boolean ret = gconf_client_set_int(getHandle(), key, value, errorHandle);
+		checkError(errorHandle);
 		return ret;
 	}
 	
@@ -386,9 +390,9 @@ public class ConfClient extends GObject {
 	 * @throws ConfException
 	 */
 	public boolean setString(String key, String value) throws ConfException {
-		int err = -1;
-		boolean ret = gconf_client_set_string(getHandle(), key, value, new int[] {err});
-		checkError(err);
+		Handle errorHandle = Struct.getNullHandle();
+		boolean ret = gconf_client_set_string(getHandle(), key, value, errorHandle);
+		checkError(errorHandle);
 		return ret;
 	}
 	
@@ -400,9 +404,9 @@ public class ConfClient extends GObject {
 	 * @throws ConfException
 	 */
 	public boolean setBoolean(String key, boolean value) throws ConfException {
-		int err = -1;
-		boolean ret = gconf_client_set_bool(getHandle(), key, value, new int[] {err});
-		checkError(err);
+		Handle errorHandle = Struct.getNullHandle();
+		boolean ret = gconf_client_set_bool(getHandle(), key, value, errorHandle);
+		checkError(errorHandle);
 		return ret;
 	}
 	
@@ -414,9 +418,9 @@ public class ConfClient extends GObject {
 	 * @throws ConfException
 	 */
 	public boolean setSchema(String key, ConfSchema value) throws ConfException {
-		int err = -1;
-		boolean ret = gconf_client_set_schema(getHandle(), key, value.getHandle(), new int[] {err});
-		checkError(err);
+		Handle errorHandle = Struct.getNullHandle();
+		boolean ret = gconf_client_set_schema(getHandle(), key, value.getHandle(), errorHandle);
+		checkError(errorHandle);
 		return ret;
 	}
 	
@@ -426,6 +430,9 @@ public class ConfClient extends GObject {
 	}
 	
 	protected void notifyCallback(Handle e, String nameSpace) {
+        if (listeners == null) {
+            return;
+        }
 		NotifyData nd = (NotifyData)listeners.get(nameSpace);
 		if (null == nd)
 			return;
@@ -439,48 +446,49 @@ public class ConfClient extends GObject {
 		}
 	}
 	
-	private void checkError(int error) throws ConfException {
-		if (-1 < error)
-			throw new ConfException(error);
+	private void checkError(Handle errorHandle) throws ConfException {
+		if (errorHandle.isNull() == false) {
+            Error error = new Error(errorHandle);
+            throw new ConfException(error.getErrorCode());
+        }
 	}
 
-
 	native static final protected Handle gconf_client_get_default();
 	native static final protected Handle gconf_client_get_for_engine(Handle engine);
-	native static final protected void gconf_client_add_dir(Handle client, String dir, int preloadType, int[] error);
-	native static final protected void gconf_client_remove_dir(Handle client, String dir, int[] error);
-	native final protected int gconf_client_notify_add(Handle client, String namespace, int[] error);
+	native static final protected void gconf_client_add_dir(Handle client, String dir, int preloadType, Handle error);
+	native static final protected void gconf_client_remove_dir(Handle client, String dir, Handle error);
+	native final protected int gconf_client_notify_add(Handle client, String namespace, Handle error);
 	native static protected void gconf_client_notify_remove(Handle client, int cnxn);
 	native static final protected void gconf_client_clear_cache(Handle client);
-	native static final protected void gconf_client_preload(Handle client, String dirname, int type, int[] error);
-	native static final protected void gconf_client_set(Handle client, String key, Handle val, int[] error);
-	native static final protected Handle gconf_client_get(Handle client, String key, int[] error);
-	native static final protected Handle gconf_client_get_without_default(Handle client, String key, int[] error);
-	native static final protected Handle gconf_client_get_entry(Handle client, String key, String locale, boolean useSchemaDefault, int[] error);
-	native static final protected Handle gconf_client_get_default_from_schema(Handle client, String key, int[] error);
-	native static final protected boolean gconf_client_unset(Handle client, String key, int[] error);
-	native static final protected Handle[] gconf_client_all_entries(Handle client, String dir, int[] error);
-	native static final protected String[] gconf_client_all_dirs(Handle client, String dir, int[] error);
-	native static final protected void gconf_client_suggest_sync(Handle client, int[] error);
-	native static final protected boolean gconf_client_dir_exists(Handle client, String dir, int[] error);
-	native static final protected double gconf_client_get_float(Handle client, String key, int[] error);
-	native static final protected int gconf_client_get_int(Handle client, String key, int[] error);
-	native static final protected String gconf_client_get_string(Handle client, String key, int[] error);
-	native static final protected boolean gconf_client_get_bool(Handle client, String key, int[] error);
-	native static final protected Handle gconf_client_get_schema(Handle client, String key, int[] error);
-	native static final protected Object[] gconf_client_get_list(Handle client, String key, int listType, int[] error);
-	native static final protected boolean gconf_client_get_pair(Handle client, String key, int carType, int cdrType, int[] car, int[] cdr, int[] error);
-	native static final protected boolean gconf_client_set_float(Handle client, String key, double val, int[] error);
-	native static final protected boolean gconf_client_set_int(Handle client, String key, int val, int[] error);
-	native static final protected boolean gconf_client_set_string(Handle client, String key, String val, int[] error);
-	native static final protected boolean gconf_client_set_bool(Handle client, String key, boolean val, int[] error);
-	native static final protected boolean gconf_client_set_schema(Handle client, String key, Handle val, int[] error);
-	native static final protected boolean gconf_client_set_list(Handle client, String key, int listType, Object[] list, int[] error);
-	native static final protected boolean gconf_client_set_pair(Handle client, String key, int carType, int cdrType, int car, int cdr, int[] error);
+	native static final protected void gconf_client_preload(Handle client, String dirname, int type, Handle error);
+	native static final protected void gconf_client_set(Handle client, String key, Handle val, Handle error);
+	native static final protected Handle gconf_client_get(Handle client, String key, Handle error);
+	native static final protected Handle gconf_client_get_without_default(Handle client, String key, Handle error);
+	native static final protected Handle gconf_client_get_entry(Handle client, String key, String locale, boolean useSchemaDefault, Handle error);
+	native static final protected Handle gconf_client_get_default_from_schema(Handle client, String key, Handle error);
+	native static final protected boolean gconf_client_unset(Handle client, String key, Handle error);
+	native static final protected Handle[] gconf_client_all_entries(Handle client, String dir, Handle error);
+	native static final protected String[] gconf_client_all_dirs(Handle client, String dir, Handle error);
+	native static final protected void gconf_client_suggest_sync(Handle client, Handle error);
+	native static final protected boolean gconf_client_dir_exists(Handle client, String dir, Handle error);
+	native static final protected double gconf_client_get_float(Handle client, String key, Handle error);
+	native static final protected int gconf_client_get_int(Handle client, String key, Handle error);
+	native static final protected String gconf_client_get_string(Handle client, String key, Handle error);
+	native static final protected boolean gconf_client_get_bool(Handle client, String key, Handle error);
+	native static final protected Handle gconf_client_get_schema(Handle client, String key, Handle error);
+	native static final protected Object[] gconf_client_get_list(Handle client, String key, int listType, Handle error);
+	native static final protected boolean gconf_client_get_pair(Handle client, String key, int carType, int cdrType, int[] car, int[] cdr, Handle error);
+	native static final protected boolean gconf_client_set_float(Handle client, String key, double val, Handle error);
+	native static final protected boolean gconf_client_set_int(Handle client, String key, int val, Handle error);
+	native static final protected boolean gconf_client_set_string(Handle client, String key, String val, Handle error);
+	native static final protected boolean gconf_client_set_bool(Handle client, String key, boolean val, Handle error);
+	native static final protected boolean gconf_client_set_schema(Handle client, String key, Handle val, Handle error);
+	native static final protected boolean gconf_client_set_list(Handle client, String key, int listType, Object[] list, Handle error);
+	native static final protected boolean gconf_client_set_pair(Handle client, String key, int carType, int cdrType, int car, int cdr, Handle error);
 	native static final protected void gconf_client_value_changed(Handle client, String key, Handle value);
 
 
 	static {
-		System.loadLibrary("gconfjni-2.9");
+		System.loadLibrary(Config.LIBRARY_NAME + Config.GCONF_API_VERSION);
 	}
 }
